package org.vamsas.client.picking;

import java.net.*;
import java.util.*;

public class PickManager
{
	// Maintains a list of client communication objects - each object represents
	// a way of talking to either:
	//  the server - if this is client side (and in which case, the list will only contain one element
	//  the other clients - if this is server side
	private LinkedList clients;
	
	private PickServer server;
	
	public PickManager()
	{
		server = new PickServer(this);
		clients = new LinkedList();
		
		new InitializeThread().start();
	}
	
	// Called whenver we try to make a new endpoint<->endpoint connection
	boolean addClientConnection(Socket socket)
	{
		PickEndPoint client = new PickEndPoint(this, socket);
		
		if (client.openConnection())
		{
			clients.add(client);
			System.out.println("List now contains " + clients.size() + " client(s)");
			return true;
		}
		
		return false;
	}
	
	public void sendMessage(String str)
	{
		forwardMessage(null, str);
	}
	
	// Forwards (or sends) a message
	// When the server receives a message, it will be resent to all clients,
	// but *not* to the client that sent it out in the first place!
	private void forwardMessage(PickEndPoint origin, String str)
	{
		ListIterator itor = clients.listIterator();
		while (itor.hasNext())
		{
			PickEndPoint client = (PickEndPoint) itor.next();
			
			if (client != origin)
				client.send(str);
		}
	}
	
	// Called by the endpoint code when a message is received
	// The manager has to:
	//  a) hand message to VAMSAS app
	//  b) forward it to other clients if we're the server
	void handleMessage(PickEndPoint origin, String str)
	{
		if (server.isServer())
			forwardMessage(origin, str);
		
		// TODO: pass message to VAMSAS API
	}
	
	void removeEndPoint(PickEndPoint client)
	{
		clients.remove(client);
		System.out.println("List now contains " + clients.size() + " client(s)");
		
		// If there's no endpoints left, then we've lost all connections and
		// need to reinitialize
		if (clients.size() == 0)
			new InitializeThread().start();
	}
	
	private class InitializeThread extends Thread
	{
		public void run()
		{
			System.out.println("Initializing connection...");
			boolean connected = false;
			
			// Loop until we can get a connection (one way or the other)
			while (!connected)
			{
				// Sleep for a rnd time so we don't end up with all the VAMSAS
				// apps trying to initialize servers at the same time
				try { Thread.sleep((int)Math.random()); }
				catch (InterruptedException e) {}
					
				// Attempt to open the server port...
				if (server.isServer() || server.createServer())
					connected = true;

				// If it fails, then attempt to make a client connection...
				else if (addClientConnection(null))
					connected = true;
			}
		}
	}
}